from PyObjCTools.TestSupport import TestCase

import MLCompute


class TestMLCMatMulDescriptor(TestCase):
    def test_methods(self):
        self.assertResultIsBOOL(MLCompute.MLCMatMulDescriptor.transposesX)
        self.assertResultIsBOOL(MLCompute.MLCMatMulDescriptor.transposesY)

        self.assertArgIsBOOL(
            MLCompute.MLCMatMulDescriptor.descriptorWithAlpha_transposesX_transposesY_,
            1,
        )
        self.assertArgIsBOOL(
            MLCompute.MLCMatMulDescriptor.descriptorWithAlpha_transposesX_transposesY_,
            2,
        )



from PyObjCTools.TestSupport import TestCase

import MLCompute


class TestMLCEmbeddingDescriptor(TestCase):
    def test_methods(self):
        self.assertResultIsBOOL(
            MLCompute.MLCEmbeddingDescriptor.scalesGradientByFrequency
        )

        self.assertArgIsBOOL(
            MLCompute.MLCEmbeddingDescriptor.descriptorWithEmbeddingCount_embeddingDimension_paddingIndex_maximumNorm_pNorm_scalesGradientByFrequency_,  # noqa: B950
            5,
        )



from PyObjCTools.TestSupport import TestCase

import MLCompute


class TestMLCConvolutionDescriptor(TestCase):
    def test_methods(self):
        self.assertResultIsBOOL(
            MLCompute.MLCConvolutionDescriptor.isConvolutionTranspose
        )
        self.assertResultIsBOOL(
            MLCompute.MLCConvolutionDescriptor.usesDepthwiseConvolution
        )



from PyObjCTools.TestSupport import TestCase

import MLCompute


class TestMLCTensorDescriptor(TestCase):
    def test_methods(self):
        self.assertResultIsBOOL(MLCompute.MLCTensorDescriptor.sortedSequences)



from PyObjCTools.TestSupport import TestCase, min_os_level

import MLCompute


class TestMLCAdamWOptimizer(TestCase):
    @min_os_level("12.0")
    def test_methods12_0(self):
        self.assertResultIsBOOL(MLCompute.MLCAdamWOptimizer.usesAMSGrad)

        self.assertArgIsBOOL(
            MLCompute.MLCAdamWOptimizer.optimizerWithDescriptor_beta1_beta2_epsilon_usesAMSGrad_timeStep_,
            4,
        )



from PyObjCTools.TestSupport import TestCase, min_os_level

import MLCompute


class TestMLAdamOptimzer(TestCase):
    @min_os_level("12.0")
    def test_methods12_0(self):
        self.assertResultIsBOOL(MLCompute.MLCAdamOptimizer.usesAMSGrad)

        self.assertArgIsBOOL(
            MLCompute.MLCAdamOptimizer.optimizerWithDescriptor_beta1_beta2_epsilon_usesAMSGrad_timeStep_,
            4,
        )



from PyObjCTools.TestSupport import TestCase

import MLCompute


class TestMLCPoolingDescriptor(TestCase):
    def test_methods(self):
        self.assertResultIsBOOL(MLCompute.MLCPoolingDescriptor.countIncludesPadding)
        self.assertArgIsBOOL(
            MLCompute.MLCPoolingDescriptor.averagePoolingDescriptorWithKernelSizes_strides_dilationRates_paddingPolicy_paddingSizes_countIncludesPadding_,  # noqa: B950
            5,
        )



from PyObjCTools.TestSupport import TestCase

import MLCompute


class TestMLCTensorParameter(TestCase):
    def test_methods(self):
        self.assertResultIsBOOL(MLCompute.MLCTensorParameter.isUpdatable)
        self.assertArgIsBOOL(MLCompute.MLCTensorParameter.setIsUpdatable_, 0)



from PyObjCTools.TestSupport import TestCase

import MLCompute


class TestMLCLayer(TestCase):
    def test_methods(self):
        self.assertResultIsBOOL(MLCompute.MLCLayer.isDebuggingEnabled)

        self.assertResultIsBOOL(MLCompute.MLCLayer.supportsDataType_onDevice_)



from PyObjCTools.TestSupport import TestCase, min_os_level

import MLCompute


class TestMLCOptimizer(TestCase):
    def test_methods(self):
        self.assertResultIsBOOL(MLCompute.MLCOptimizer.appliesGradientClipping)

    @min_os_level("12.0")
    def test_methods12_0(self):
        self.assertArgIsBOOL(
            MLCompute.MLCOptimizerDescriptor.descriptorWithLearningRate_gradientRescale_appliesGradientClipping_gradientClippingType_gradientClipMax_gradientClipMin_maximumClippingNorm_customGlobalNorm_regularizationType_regularizationScale_,  # noqa: B950
            2,
        )



from PyObjCTools.TestSupport import TestCase

import MLCompute


class TestMLCMultiheadAttentionDescriptor(TestCase):
    def test_methods(self):
        self.assertResultIsBOOL(MLCompute.MLCMultiheadAttentionDescriptor.hasBiases)
        self.assertResultIsBOOL(
            MLCompute.MLCMultiheadAttentionDescriptor.hasAttentionBiases
        )
        self.assertResultIsBOOL(
            MLCompute.MLCMultiheadAttentionDescriptor.addsZeroAttention
        )

        self.assertArgIsBOOL(
            MLCompute.MLCMultiheadAttentionDescriptor.descriptorWithModelDimension_keyDimension_valueDimension_headCount_dropout_hasBiases_hasAttentionBiases_addsZeroAttention_,  # noqa: B950
            5,
        )
        self.assertArgIsBOOL(
            MLCompute.MLCMultiheadAttentionDescriptor.descriptorWithModelDimension_keyDimension_valueDimension_headCount_dropout_hasBiases_hasAttentionBiases_addsZeroAttention_,  # noqa: B950
            6,
        )
        self.assertArgIsBOOL(
            MLCompute.MLCMultiheadAttentionDescriptor.descriptorWithModelDimension_keyDimension_valueDimension_headCount_dropout_hasBiases_hasAttentionBiases_addsZeroAttention_,  # noqa: B950
            7,
        )



from PyObjCTools.TestSupport import TestCase, min_os_level

import MLCompute


class TestMLCDevice(TestCase):
    @min_os_level("10.15")
    def test_methods(self):
        self.assertArgIsBOOL(
            MLCompute.MLCDevice.deviceWithType_selectsMultipleComputeDevices_, 1
        )



from PyObjCTools.TestSupport import TestCase

import MLCompute


class TestMLCTensor(TestCase):
    def test_methods(self):
        self.assertArgIsBOOL(
            MLCompute.MLCTensor.tensorWithSequenceLengths_sortedSequences_featureChannelCount_batchSize_randomInitializerType_,
            1,
        )
        self.assertArgIsBOOL(
            MLCompute.MLCTensor.tensorWithSequenceLengths_sortedSequences_featureChannelCount_batchSize_data_,
            1,
        )

        self.assertResultIsBOOL(MLCompute.MLCTensor.hasValidNumerics)
        self.assertResultIsBOOL(MLCompute.MLCTensor.synchronizeData)

        self.assertResultIsBOOL(
            MLCompute.MLCTensor.copyDataFromDeviceMemoryToBytes_length_synchronizeWithDevice_
        )
        self.assertArgIsBOOL(
            MLCompute.MLCTensor.copyDataFromDeviceMemoryToBytes_length_synchronizeWithDevice_,
            2,
        )

        self.assertResultIsBOOL(MLCompute.MLCTensor.bindAndWriteData_toDevice_)

        self.assertResultIsBOOL(MLCompute.MLCTensor.synchronizeOptimizerData)
        self.assertResultIsBOOL(MLCompute.MLCTensor.bindOptimizerData_deviceData_)



from PyObjCTools.TestSupport import TestCase

import MLCompute


class TestMLCUpsampleLayer(TestCase):
    def test_methods(self):
        self.assertResultIsBOOL(MLCompute.MLCUpsampleLayer.alignsCorners)

        self.assertArgIsBOOL(
            MLCompute.MLCUpsampleLayer.layerWithShape_sampleMode_alignsCorners_, 2
        )



from PyObjCTools.TestSupport import TestCase, min_os_level

import MLCompute


class TestMLCTrainingGraph(TestCase):
    def test_methods(self):
        self.assertResultIsBOOL(MLCompute.MLCTrainingGraph.addInputs_lossLabels_)
        self.assertResultIsBOOL(
            MLCompute.MLCTrainingGraph.addInputs_lossLabels_lossLabelWeights_
        )
        self.assertResultIsBOOL(MLCompute.MLCTrainingGraph.addOutputs_)
        self.assertResultIsBOOL(MLCompute.MLCTrainingGraph.stopGradientForTensors_)
        self.assertResultIsBOOL(MLCompute.MLCTrainingGraph.compileWithOptions_device_)
        self.assertResultIsBOOL(MLCompute.MLCTrainingGraph.compileOptimizer_)
        self.assertResultIsBOOL(MLCompute.MLCTrainingGraph.linkWithGraphs_)

        self.assertResultIsBOOL(
            MLCompute.MLCTrainingGraph.executeWithInputsData_lossLabelsData_lossLabelWeightsData_batchSize_options_completionHandler_  # noqa: B950
        )
        self.assertArgIsBlock(
            MLCompute.MLCTrainingGraph.executeWithInputsData_lossLabelsData_lossLabelWeightsData_batchSize_options_completionHandler_,  # noqa: B950
            5,
            b"v@@d",
        )

        self.assertResultIsBOOL(
            MLCompute.MLCTrainingGraph.executeWithInputsData_lossLabelsData_lossLabelWeightsData_outputsData_batchSize_options_completionHandler_  # noqa: B950
        )
        self.assertArgIsBlock(
            MLCompute.MLCTrainingGraph.executeWithInputsData_lossLabelsData_lossLabelWeightsData_outputsData_batchSize_options_completionHandler_,  # noqa: B950
            6,
            b"v@@d",
        )

        self.assertResultIsBOOL(
            MLCompute.MLCTrainingGraph.executeForwardWithBatchSize_options_completionHandler_
        )
        self.assertArgIsBlock(
            MLCompute.MLCTrainingGraph.executeForwardWithBatchSize_options_completionHandler_,
            2,
            b"v@@d",
        )

        self.assertResultIsBOOL(
            MLCompute.MLCTrainingGraph.executeForwardWithBatchSize_options_outputsData_completionHandler_
        )
        self.assertArgIsBlock(
            MLCompute.MLCTrainingGraph.executeForwardWithBatchSize_options_outputsData_completionHandler_,
            3,
            b"v@@d",
        )

        self.assertResultIsBOOL(
            MLCompute.MLCTrainingGraph.executeGradientWithBatchSize_options_completionHandler_
        )
        self.assertArgIsBlock(
            MLCompute.MLCTrainingGraph.executeGradientWithBatchSize_options_completionHandler_,
            2,
            b"v@@d",
        )

        self.assertResultIsBOOL(
            MLCompute.MLCTrainingGraph.executeGradientWithBatchSize_options_outputsData_completionHandler_
        )
        self.assertArgIsBlock(
            MLCompute.MLCTrainingGraph.executeGradientWithBatchSize_options_outputsData_completionHandler_,
            3,
            b"v@@d",
        )

        self.assertResultIsBOOL(MLCompute.MLCTrainingGraph.setTrainingTensorParameters_)

        self.assertResultIsBOOL(
            MLCompute.MLCTrainingGraph.bindOptimizerData_deviceData_withTensor_
        )

    @min_os_level("11.3")
    def test_methods11_3(self):
        self.assertResultIsBOOL(
            MLCompute.MLCTrainingGraph.compileWithOptions_device_inputTensors_inputTensorsData_
        )



# This file is generated by objective.metadata
#
# Last update: Sun Aug  1 09:52:23 2021
#
# flake8: noqa

import objc, sys

if sys.maxsize > 2 ** 32:

    def sel32or64(a, b):
        return b


else:

    def sel32or64(a, b):
        return a


if objc.arch == "arm64":

    def selAorI(a, b):
        return a


else:

    def selAorI(a, b):
        return b


misc = {}
constants = """$$"""
enums = """$MLCActivationTypeAbsolute@6$MLCActivationTypeCELU@13$MLCActivationTypeClamp@20$MLCActivationTypeELU@9$MLCActivationTypeGELU@18$MLCActivationTypeHardShrink@14$MLCActivationTypeHardSigmoid@4$MLCActivationTypeHardSwish@19$MLCActivationTypeLinear@2$MLCActivationTypeLogSigmoid@11$MLCActivationTypeNone@0$MLCActivationTypeReLU@1$MLCActivationTypeReLUN@10$MLCActivationTypeSELU@12$MLCActivationTypeSigmoid@3$MLCActivationTypeSoftPlus@7$MLCActivationTypeSoftShrink@15$MLCActivationTypeSoftSign@8$MLCActivationTypeTanh@5$MLCActivationTypeTanhShrink@16$MLCActivationTypeThreshold@17$MLCArithmeticOperationAcos@13$MLCArithmeticOperationAcosh@19$MLCArithmeticOperationAdd@0$MLCArithmeticOperationAsin@12$MLCArithmeticOperationAsinh@18$MLCArithmeticOperationAtan@14$MLCArithmeticOperationAtanh@20$MLCArithmeticOperationCeil@6$MLCArithmeticOperationCos@10$MLCArithmeticOperationCosh@16$MLCArithmeticOperationDivide@3$MLCArithmeticOperationDivideNoNaN@27$MLCArithmeticOperationExp@22$MLCArithmeticOperationExp2@23$MLCArithmeticOperationFloor@4$MLCArithmeticOperationLog@24$MLCArithmeticOperationLog2@25$MLCArithmeticOperationMax@29$MLCArithmeticOperationMin@28$MLCArithmeticOperationMultiply@2$MLCArithmeticOperationMultiplyNoNaN@26$MLCArithmeticOperationPow@21$MLCArithmeticOperationRound@5$MLCArithmeticOperationRsqrt@8$MLCArithmeticOperationSin@9$MLCArithmeticOperationSinh@15$MLCArithmeticOperationSqrt@7$MLCArithmeticOperationSubtract@1$MLCArithmeticOperationTan@11$MLCArithmeticOperationTanh@17$MLCComparisonOperationEqual@0$MLCComparisonOperationGreater@3$MLCComparisonOperationGreaterOrEqual@5$MLCComparisonOperationLess@2$MLCComparisonOperationLessOrEqual@4$MLCComparisonOperationLogicalAND@6$MLCComparisonOperationLogicalNAND@9$MLCComparisonOperationLogicalNOR@10$MLCComparisonOperationLogicalNOT@8$MLCComparisonOperationLogicalOR@7$MLCComparisonOperationLogicalXOR@11$MLCComparisonOperationNotEqual@1$MLCConvolutionTypeDepthwise@2$MLCConvolutionTypeStandard@0$MLCConvolutionTypeTransposed@1$MLCDataTypeBoolean@4$MLCDataTypeFloat16@3$MLCDataTypeFloat32@1$MLCDataTypeInt32@7$MLCDataTypeInt64@5$MLCDataTypeInt8@8$MLCDataTypeInvalid@0$MLCDataTypeUInt8@9$MLCDeviceTypeANE@3$MLCDeviceTypeAny@2$MLCDeviceTypeCPU@0$MLCDeviceTypeGPU@1$MLCExecutionOptionsForwardForInference@8$MLCExecutionOptionsForwardOnlyForInference@8$MLCExecutionOptionsNone@0$MLCExecutionOptionsPerLayerProfiling@16$MLCExecutionOptionsProfiling@4$MLCExecutionOptionsSkipWritingInputDataToDevice@1$MLCExecutionOptionsSynchronous@2$MLCGradientClippingTypeByGlobalNorm@2$MLCGradientClippingTypeByNorm@1$MLCGradientClippingTypeByValue@0$MLCGraphCompilationOptionsComputeAllGradients@8$MLCGraphCompilationOptionsDebugLayers@1$MLCGraphCompilationOptionsDisableLayerFusion@2$MLCGraphCompilationOptionsLinkGraphs@4$MLCGraphCompilationOptionsNone@0$MLCLSTMResultModeOutput@0$MLCLSTMResultModeOutputAndStates@1$MLCLossTypeCategoricalCrossEntropy@4$MLCLossTypeCosineDistance@7$MLCLossTypeHinge@5$MLCLossTypeHuber@6$MLCLossTypeLog@8$MLCLossTypeMeanAbsoluteError@0$MLCLossTypeMeanSquaredError@1$MLCLossTypeSigmoidCrossEntropy@3$MLCLossTypeSoftmaxCrossEntropy@2$MLCPaddingPolicySame@0$MLCPaddingPolicyUsePaddingSize@2$MLCPaddingPolicyValid@1$MLCPaddingTypeConstant@3$MLCPaddingTypeReflect@1$MLCPaddingTypeSymmetric@2$MLCPaddingTypeZero@0$MLCPoolingTypeAverage@2$MLCPoolingTypeL2Norm@3$MLCPoolingTypeMax@1$MLCRandomInitializerTypeGlorotUniform@2$MLCRandomInitializerTypeInvalid@0$MLCRandomInitializerTypeUniform@1$MLCRandomInitializerTypeXavier@3$MLCReductionTypeAll@9$MLCReductionTypeAny@8$MLCReductionTypeArgMax@5$MLCReductionTypeArgMin@6$MLCReductionTypeL1Norm@7$MLCReductionTypeMax@3$MLCReductionTypeMean@2$MLCReductionTypeMin@4$MLCReductionTypeNone@0$MLCReductionTypeSum@1$MLCRegularizationTypeL1@1$MLCRegularizationTypeL2@2$MLCRegularizationTypeNone@0$MLCSampleModeLinear@1$MLCSampleModeNearest@0$MLCSoftmaxOperationLogSoftmax@1$MLCSoftmaxOperationSoftmax@0$"""
misc.update({})
functions = {
    "MLCPaddingTypeDebugDescription": (b"@i",),
    "MLCReductionTypeDebugDescription": (b"@i",),
    "MLCLossTypeDebugDescription": (b"@i",),
    "MLCGradientClippingTypeDebugDescription": (b"@i",),
    "MLCComparisonOperationDebugDescription": (b"@i",),
    "MLCLSTMResultModeDebugDescription": (b"@Q",),
    "MLCPoolingTypeDebugDescription": (b"@i",),
    "MLCActivationTypeDebugDescription": (b"@i",),
    "MLCPaddingPolicyDebugDescription": (b"@i",),
    "MLCConvolutionTypeDebugDescription": (b"@i",),
    "MLCSoftmaxOperationDebugDescription": (b"@i",),
    "MLCArithmeticOperationDebugDescription": (b"@i",),
    "MLCSampleModeDebugDescription": (b"@i",),
}
r = objc.registerMetaDataForSelector
objc._updatingMetadata(True)
try:
    r(
        b"MLCAdamOptimizer",
        b"optimizerWithDescriptor:beta1:beta2:epsilon:usesAMSGrad:timeStep:",
        {"arguments": {6: {"type": b"Z"}}},
    )
    r(b"MLCAdamOptimizer", b"usesAMSGrad", {"retval": {"type": b"Z"}})
    r(
        b"MLCAdamWOptimizer",
        b"optimizerWithDescriptor:beta1:beta2:epsilon:usesAMSGrad:timeStep:",
        {"arguments": {6: {"type": b"Z"}}},
    )
    r(b"MLCAdamWOptimizer", b"usesAMSGrad", {"retval": {"type": b"Z"}})
    r(
        b"MLCConvolutionDescriptor",
        b"isConvolutionTranspose",
        {"retval": {"type": b"Z"}},
    )
    r(
        b"MLCConvolutionDescriptor",
        b"usesDepthwiseConvolution",
        {"retval": {"type": b"Z"}},
    )
    r(
        b"MLCDevice",
        b"deviceWithType:selectsMultipleComputeDevices:",
        {"arguments": {3: {"type": b"Z"}}},
    )
    r(
        b"MLCEmbeddingDescriptor",
        b"descriptorWithEmbeddingCount:embeddingDimension:paddingIndex:maximumNorm:pNorm:scalesGradientByFrequency:",
        {"arguments": {7: {"type": b"Z"}}},
    )
    r(
        b"MLCEmbeddingDescriptor",
        b"scalesGradientByFrequency",
        {"retval": {"type": b"Z"}},
    )
    r(
        b"MLCGraph",
        b"bindAndWriteData:forInputs:toDevice:batchSize:synchronous:",
        {"retval": {"type": b"Z"}, "arguments": {6: {"type": b"Z"}}},
    )
    r(
        b"MLCGraph",
        b"bindAndWriteData:forInputs:toDevice:synchronous:",
        {"retval": {"type": "Z"}, "arguments": {5: {"type": "Z"}}},
    )
    r(
        b"MLCGraph",
        b"nodeWithLayer:sources:disableUpdate:",
        {"arguments": {4: {"type": b"Z"}}},
    )
    r(b"MLCInferenceGraph", b"addInputs:", {"retval": {"type": b"Z"}})
    r(
        b"MLCInferenceGraph",
        b"addInputs:lossLabels:lossLabelWeights:",
        {"retval": {"type": b"Z"}},
    )
    r(b"MLCInferenceGraph", b"addOutputs:", {"retval": {"type": b"Z"}})
    r(b"MLCInferenceGraph", b"compileWithOptions:device:", {"retval": {"type": b"Z"}})
    r(
        b"MLCInferenceGraph",
        b"compileWithOptions:device:inputTensors:inputTensorsData:",
        {"retval": {"type": "Z"}},
    )
    r(
        b"MLCInferenceGraph",
        b"executeWithInputsData:batchSize:options:completionHandler:",
        {
            "retval": {"type": b"Z"},
            "arguments": {
                5: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                            3: {"type": b"d"},
                        },
                    }
                }
            },
        },
    )
    r(
        b"MLCInferenceGraph",
        b"executeWithInputsData:lossLabelsData:lossLabelWeightsData:batchSize:options:completionHandler:",
        {
            "retval": {"type": b"Z"},
            "arguments": {
                7: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                            3: {"type": b"d"},
                        },
                    }
                }
            },
        },
    )
    r(
        b"MLCInferenceGraph",
        b"executeWithInputsData:lossLabelsData:lossLabelWeightsData:outputsData:batchSize:options:completionHandler:",
        {
            "retval": {"type": b"Z"},
            "arguments": {
                8: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                            3: {"type": b"d"},
                        },
                    }
                }
            },
        },
    )
    r(
        b"MLCInferenceGraph",
        b"executeWithInputsData:outputsData:batchSize:options:completionHandler:",
        {
            "retval": {"type": b"Z"},
            "arguments": {
                6: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                            3: {"type": b"d"},
                        },
                    }
                }
            },
        },
    )
    r(b"MLCInferenceGraph", b"linkWithGraphs:", {"retval": {"type": b"Z"}})
    r(b"MLCLSTMDescriptor", b"batchFirst", {"retval": {"type": b"Z"}})
    r(
        b"MLCLSTMDescriptor",
        b"descriptorWithInputSize:hiddenSize:layerCount:usesBiases:batchFirst:isBidirectional:dropout:",
        {"arguments": {5: {"type": b"Z"}, 6: {"type": b"Z"}, 7: {"type": b"Z"}}},
    )
    r(
        b"MLCLSTMDescriptor",
        b"descriptorWithInputSize:hiddenSize:layerCount:usesBiases:batchFirst:isBidirectional:returnsSequences:dropout:",
        {
            "arguments": {
                5: {"type": b"Z"},
                6: {"type": b"Z"},
                7: {"type": b"Z"},
                8: {"type": b"Z"},
            }
        },
    )
    r(
        b"MLCLSTMDescriptor",
        b"descriptorWithInputSize:hiddenSize:layerCount:usesBiases:batchFirst:isBidirectional:returnsSequences:dropout:resultMode:",
        {
            "arguments": {
                5: {"type": "Z"},
                6: {"type": "Z"},
                7: {"type": "Z"},
                8: {"type": "Z"},
            }
        },
    )
    r(
        b"MLCLSTMDescriptor",
        b"descriptorWithInputSize:hiddenSize:layerCount:usesBiases:isBidirectional:dropout:",
        {"arguments": {5: {"type": b"Z"}, 6: {"type": b"Z"}}},
    )
    r(b"MLCLSTMDescriptor", b"isBidirectional", {"retval": {"type": b"Z"}})
    r(b"MLCLSTMDescriptor", b"returnsSequences", {"retval": {"type": b"Z"}})
    r(b"MLCLSTMDescriptor", b"usesBiases", {"retval": {"type": b"Z"}})
    r(b"MLCLayer", b"isDebuggingEnabled", {"retval": {"type": b"Z"}})
    r(b"MLCLayer", b"setIsDebuggingEnabled:", {"arguments": {2: {"type": b"Z"}}})
    r(b"MLCLayer", b"supportsDataType:onDevice:", {"retval": {"type": "Z"}})
    r(
        b"MLCMatMulDescriptor",
        b"descriptorWithAlpha:transposesX:transposesY:",
        {"arguments": {3: {"type": b"Z"}, 4: {"type": b"Z"}}},
    )
    r(b"MLCMatMulDescriptor", b"transposesX", {"retval": {"type": b"Z"}})
    r(b"MLCMatMulDescriptor", b"transposesY", {"retval": {"type": b"Z"}})
    r(
        b"MLCMultiheadAttentionDescriptor",
        b"addsZeroAttention",
        {"retval": {"type": b"Z"}},
    )
    r(
        b"MLCMultiheadAttentionDescriptor",
        b"descriptorWithModelDimension:keyDimension:valueDimension:headCount:dropout:hasBiases:hasAttentionBiases:addsZeroAttention:",
        {"arguments": {7: {"type": b"Z"}, 8: {"type": b"Z"}, 9: {"type": b"Z"}}},
    )
    r(
        b"MLCMultiheadAttentionDescriptor",
        b"hasAttentionBiases",
        {"retval": {"type": b"Z"}},
    )
    r(b"MLCMultiheadAttentionDescriptor", b"hasBiases", {"retval": {"type": b"Z"}})
    r(b"MLCOptimizer", b"appliesGradientClipping", {"retval": {"type": b"Z"}})
    r(
        b"MLCOptimizer",
        b"setAppliesGradientClipping:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(b"MLCOptimizerDescriptor", b"appliesGradientClipping", {"retval": {"type": b"Z"}})
    r(
        b"MLCOptimizerDescriptor",
        b"descriptorWithLearningRate:gradientRescale:appliesGradientClipping:gradientClipMax:gradientClipMin:regularizationType:regularizationScale:",
        {"arguments": {4: {"type": b"Z"}}},
    )
    r(
        b"MLCOptimizerDescriptor",
        b"descriptorWithLearningRate:gradientRescale:appliesGradientClipping:gradientClippingType:gradientClipMax:gradientClipMin:maximumClippingNorm:customGlobalNorm:regularizationType:regularizationScale:",
        {"arguments": {4: {"type": b"Z"}}},
    )
    r(
        b"MLCPoolingDescriptor",
        b"averagePoolingDescriptorWithKernelSizes:strides:dilationRates:paddingPolicy:paddingSizes:countIncludesPadding:",
        {"arguments": {7: {"type": b"Z"}}},
    )
    r(
        b"MLCPoolingDescriptor",
        b"averagePoolingDescriptorWithKernelSizes:strides:paddingPolicy:paddingSizes:countIncludesPadding:",
        {"arguments": {6: {"type": b"Z"}}},
    )
    r(b"MLCPoolingDescriptor", b"countIncludesPadding", {"retval": {"type": b"Z"}})
    r(b"MLCRMSPropOptimizer", b"isCentered", {"retval": {"type": b"Z"}})
    r(
        b"MLCRMSPropOptimizer",
        b"optimizerWithDescriptor:momentumScale:alpha:epsilon:isCentered:",
        {"arguments": {6: {"type": b"Z"}}},
    )
    r(
        b"MLCSGDOptimizer",
        b"optimizerWithDescriptor:momentumScale:usesNesterovMomentum:",
        {"arguments": {4: {"type": "Z"}}},
    )
    r(
        b"MLCSGDOptimizer",
        b"optimizerWithDescriptor:momentumScale:usesNestrovMomentum:",
        {"arguments": {4: {"type": b"Z"}}},
    )
    r(b"MLCSGDOptimizer", b"usesNesterovMomentum", {"retval": {"type": "Z"}})
    r(b"MLCSGDOptimizer", b"usesNestrovMomentum", {"retval": {"type": b"Z"}})
    r(b"MLCTensor", b"bindAndWriteData:toDevice:", {"retval": {"type": b"Z"}})
    r(b"MLCTensor", b"bindOptimizerData:deviceData:", {"retval": {"type": "Z"}})
    r(
        b"MLCTensor",
        b"copyDataFromDeviceMemoryToBytes:length:synchronizeWithDevice:",
        {"retval": {"type": b"Z"}, "arguments": {4: {"type": b"Z"}}},
    )
    r(b"MLCTensor", b"hasValidNumerics", {"retval": {"type": b"Z"}})
    r(b"MLCTensor", b"optimizerData:", {"retval": {"type": b"Z"}})
    r(b"MLCTensor", b"synchronizeData", {"retval": {"type": b"Z"}})
    r(b"MLCTensor", b"synchronizeOptimizerData", {"retval": {"type": "Z"}})
    r(
        b"MLCTensor",
        b"tensorWithSequenceLengths:sortedSequences:featureChannelCount:batchSize:data:",
        {"arguments": {3: {"type": b"Z"}}},
    )
    r(
        b"MLCTensor",
        b"tensorWithSequenceLengths:sortedSequences:featureChannelCount:batchSize:randomInitializerType:",
        {"arguments": {3: {"type": b"Z"}}},
    )
    r(b"MLCTensorData", b"bytes", {"retval": {"c_array_of_variable_size": True}})
    r(
        b"MLCTensorData",
        b"dataWithBytesNoCopy:length:",
        {"arguments": {2: {"type_modifier": b"n", "c_array_length_in_arg": 3}}},
    )
    r(
        b"MLCTensorData",
        b"dataWithBytesNoCopy:length:deallocator:",
        {
            "arguments": {
                2: {"type_modifier": b"n", "c_array_length_in_arg": 3},
                4: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"n^v", "c_array_length_in_arg": 2},
                            2: {"type": b"Q"},
                        },
                    }
                },
            }
        },
    )
    r(
        b"MLCTensorData",
        b"dataWithImmutableBytesNoCopy:length:",
        {"arguments": {2: {"type_modifier": b"n", "c_array_length_in_arg": 3}}},
    )
    r(
        b"MLCTensorDescriptor",
        b"descriptorWithShape:sequenceLengths:sortedSequences:dataType:",
        {"arguments": {4: {"type": b"Z"}}},
    )
    r(b"MLCTensorDescriptor", b"sortedSequences", {"retval": {"type": b"Z"}})
    r(b"MLCTensorParameter", b"isUpdatable", {"retval": {"type": b"Z"}})
    r(b"MLCTensorParameter", b"setIsUpdatable:", {"arguments": {2: {"type": b"Z"}}})
    r(b"MLCTrainingGraph", b"addInputs:lossLabels:", {"retval": {"type": b"Z"}})
    r(
        b"MLCTrainingGraph",
        b"addInputs:lossLabels:lossLabelWeights:",
        {"retval": {"type": b"Z"}},
    )
    r(b"MLCTrainingGraph", b"addOutputs:", {"retval": {"type": b"Z"}})
    r(
        b"MLCTrainingGraph",
        b"bindOptimizerData:deviceData:withTensor:",
        {"retval": {"type": "Z"}},
    )
    r(b"MLCTrainingGraph", b"compileOptimizer:", {"retval": {"type": b"Z"}})
    r(b"MLCTrainingGraph", b"compileWithOptions:device:", {"retval": {"type": b"Z"}})
    r(
        b"MLCTrainingGraph",
        b"compileWithOptions:device:inputTensors:inputTensorsData:",
        {"retval": {"type": "Z"}},
    )
    r(
        b"MLCTrainingGraph",
        b"executeForwardWithBatchSize:options:completionHandler:",
        {
            "retval": {"type": b"Z"},
            "arguments": {
                4: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                            3: {"type": b"d"},
                        },
                    }
                }
            },
        },
    )
    r(
        b"MLCTrainingGraph",
        b"executeForwardWithBatchSize:options:outputsData:completionHandler:",
        {
            "retval": {"type": b"Z"},
            "arguments": {
                5: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                            3: {"type": b"d"},
                        },
                    }
                }
            },
        },
    )
    r(
        b"MLCTrainingGraph",
        b"executeGradientWithBatchSize:options:completionHandler:",
        {
            "retval": {"type": b"Z"},
            "arguments": {
                4: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                            3: {"type": b"d"},
                        },
                    }
                }
            },
        },
    )
    r(
        b"MLCTrainingGraph",
        b"executeGradientWithBatchSize:options:outputsData:completionHandler:",
        {
            "retval": {"type": b"Z"},
            "arguments": {
                5: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                            3: {"type": b"d"},
                        },
                    }
                }
            },
        },
    )
    r(
        b"MLCTrainingGraph",
        b"executeOptimizerUpdateWithOptions:completionHandler:",
        {
            "retval": {"type": b"Z"},
            "arguments": {
                3: {
                    "callable": {
                        "retval": {"type": b"@?"},
                        "arguments": {0: {"type": b"^v"}},
                    }
                }
            },
        },
    )
    r(
        b"MLCTrainingGraph",
        b"executeWithInputsData:lossLabelsData:lossLabelWeightsData:batchSize:options:completionHandler:",
        {
            "retval": {"type": b"Z"},
            "arguments": {
                7: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                            3: {"type": b"d"},
                        },
                    }
                }
            },
        },
    )
    r(
        b"MLCTrainingGraph",
        b"executeWithInputsData:lossLabelsData:lossLabelWeightsData:outputsData:batchSize:options:completionHandler:",
        {
            "retval": {"type": b"Z"},
            "arguments": {
                8: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                            3: {"type": b"d"},
                        },
                    }
                }
            },
        },
    )
    r(b"MLCTrainingGraph", b"linkWithGraphs:", {"retval": {"type": b"Z"}})
    r(b"MLCTrainingGraph", b"setTrainingTensorParameters:", {"retval": {"type": b"Z"}})
    r(b"MLCTrainingGraph", b"stopGradientForTensors:", {"retval": {"type": b"Z"}})
    r(b"MLCUpsampleLayer", b"alignsCorners", {"retval": {"type": b"Z"}})
    r(
        b"MLCUpsampleLayer",
        b"layerWithShape:sampleMode:alignsCorners:",
        {"arguments": {4: {"type": b"Z"}}},
    )
    r(
        b"MLCYOLOLossDescriptor",
        b"setShouldRescore:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(b"MLCYOLOLossDescriptor", b"shouldRescore", {"retval": {"type": b"Z"}})
finally:
    objc._updatingMetadata(False)
expressions = {}

# END OF FILE



from PyObjCTools.TestSupport import TestCase

import MLCompute


class TestMLCSGDOptimizer(TestCase):
    def test_methods(self):
        self.assertResultIsBOOL(MLCompute.MLCSGDOptimizer.usesNesterovMomentum)

        self.assertArgIsBOOL(
            MLCompute.MLCSGDOptimizer.optimizerWithDescriptor_momentumScale_usesNesterovMomentum_,
            2,
        )



from PyObjCTools.TestSupport import TestCase

import MLCompute


class TestMLCOptimizerDescriptor(TestCase):
    def test_methods(self):
        self.assertResultIsBOOL(
            MLCompute.MLCOptimizerDescriptor.appliesGradientClipping
        )



from PyObjCTools.TestSupport import TestCase, min_os_level

import MLCompute


class TestMLCInferenceGraph(TestCase):
    def test_methods(self):
        self.assertResultIsBOOL(MLCompute.MLCInferenceGraph.addInputs_)
        self.assertResultIsBOOL(
            MLCompute.MLCInferenceGraph.addInputs_lossLabels_lossLabelWeights_
        )
        self.assertResultIsBOOL(MLCompute.MLCInferenceGraph.addOutputs_)
        self.assertResultIsBOOL(MLCompute.MLCInferenceGraph.compileWithOptions_device_)
        self.assertResultIsBOOL(MLCompute.MLCInferenceGraph.linkWithGraphs_)

        self.assertResultIsBOOL(
            MLCompute.MLCInferenceGraph.executeWithInputsData_batchSize_options_completionHandler_
        )
        self.assertArgIsBlock(
            MLCompute.MLCInferenceGraph.executeWithInputsData_batchSize_options_completionHandler_,
            3,
            b"v@@d",
        )

        self.assertResultIsBOOL(
            MLCompute.MLCInferenceGraph.executeWithInputsData_outputsData_batchSize_options_completionHandler_
        )
        self.assertArgIsBlock(
            MLCompute.MLCInferenceGraph.executeWithInputsData_outputsData_batchSize_options_completionHandler_,
            4,
            b"v@@d",
        )

        self.assertResultIsBOOL(
            MLCompute.MLCInferenceGraph.executeWithInputsData_lossLabelsData_lossLabelWeightsData_batchSize_options_completionHandler_  # noqa: B950
        )
        self.assertArgIsBlock(
            MLCompute.MLCInferenceGraph.executeWithInputsData_lossLabelsData_lossLabelWeightsData_batchSize_options_completionHandler_,  # noqa: B950
            5,
            b"v@@d",
        )

        self.assertResultIsBOOL(
            MLCompute.MLCInferenceGraph.executeWithInputsData_lossLabelsData_lossLabelWeightsData_outputsData_batchSize_options_completionHandler_  # noqa: B950
        )
        self.assertArgIsBlock(
            MLCompute.MLCInferenceGraph.executeWithInputsData_lossLabelsData_lossLabelWeightsData_outputsData_batchSize_options_completionHandler_,  # noqa: B950
            6,
            b"v@@d",
        )

    @min_os_level("11.3")
    def test_methods11_3(self):
        self.assertResultIsBOOL(
            MLCompute.MLCInferenceGraph.compileWithOptions_device_inputTensors_inputTensorsData_
        )



from PyObjCTools.TestSupport import TestCase

import MLCompute


class TestMLCYOLOLossDescriptor(TestCase):
    def test_methods(self):
        self.assertResultIsBOOL(MLCompute.MLCYOLOLossDescriptor.shouldRescore)
        self.assertArgIsBOOL(MLCompute.MLCYOLOLossDescriptor.setShouldRescore_, 0)



from PyObjCTools.TestSupport import TestCase

import MLCompute


class TestMLCRMSPropOptimizer(TestCase):
    def test_methods(self):
        self.assertResultIsBOOL(MLCompute.MLCRMSPropOptimizer.isCentered)

        self.assertArgIsBOOL(
            MLCompute.MLCRMSPropOptimizer.optimizerWithDescriptor_momentumScale_alpha_epsilon_isCentered_,
            4,
        )



from PyObjCTools.TestSupport import TestCase, min_os_level

import MLCompute
import objc


class TestMLCTensorData(TestCase):
    def test_methods(self):
        self.assertArgIsIn(MLCompute.MLCTensorData.dataWithBytesNoCopy_length_, 0)
        self.assertArgSizeInArg(
            MLCompute.MLCTensorData.dataWithBytesNoCopy_length_, 0, 1
        )

        self.assertArgIsIn(
            MLCompute.MLCTensorData.dataWithImmutableBytesNoCopy_length_, 0
        )
        self.assertArgSizeInArg(
            MLCompute.MLCTensorData.dataWithImmutableBytesNoCopy_length_, 0, 1
        )

    @min_os_level("11.3")
    def test_methods11_3(self):
        self.assertArgIsIn(
            MLCompute.MLCTensorData.dataWithBytesNoCopy_length_deallocator_, 0
        )
        self.assertArgSizeInArg(
            MLCompute.MLCTensorData.dataWithBytesNoCopy_length_deallocator_, 0, 1
        )
        self.assertArgIsBlock(
            MLCompute.MLCTensorData.dataWithBytesNoCopy_length_deallocator_,
            2,
            b"vn^v" + objc._C_NSUInteger,
        )



from PyObjCTools.TestSupport import TestCase

import MLCompute


class TestMLCGraph(TestCase):
    def test_methods(self):
        self.assertArgIsBOOL(MLCompute.MLCGraph.nodeWithLayer_sources_disableUpdate_, 2)

        self.assertResultIsBOOL(
            MLCompute.MLCGraph.bindAndWriteData_forInputs_toDevice_batchSize_synchronous_
        )
        self.assertArgIsBOOL(
            MLCompute.MLCGraph.bindAndWriteData_forInputs_toDevice_batchSize_synchronous_,
            4,
        )

        self.assertResultIsBOOL(
            MLCompute.MLCGraph.bindAndWriteData_forInputs_toDevice_synchronous_
        )
        self.assertArgIsBOOL(
            MLCompute.MLCGraph.bindAndWriteData_forInputs_toDevice_synchronous_, 3
        )



from PyObjCTools.TestSupport import TestCase, min_os_level

import MLCompute


class MLCTypes(TestCase):
    def test_constants(self):
        self.assertEqual(MLCompute.MLCDataTypeInvalid, 0)
        self.assertEqual(MLCompute.MLCDataTypeFloat32, 1)
        self.assertEqual(MLCompute.MLCDataTypeFloat16, 3)
        self.assertEqual(MLCompute.MLCDataTypeBoolean, 4)
        self.assertEqual(MLCompute.MLCDataTypeInt64, 5)
        self.assertEqual(MLCompute.MLCDataTypeInt32, 7)
        self.assertEqual(MLCompute.MLCDataTypeInt8, 8)
        self.assertNotHasAttr(MLCompute, "MLCDataTypeCount")

        self.assertEqual(MLCompute.MLCRandomInitializerTypeInvalid, 0)
        self.assertEqual(MLCompute.MLCRandomInitializerTypeUniform, 1)
        self.assertEqual(MLCompute.MLCRandomInitializerTypeGlorotUniform, 2)
        self.assertEqual(MLCompute.MLCRandomInitializerTypeXavier, 3)
        self.assertNotHasAttr(MLCompute, "MLCRandomInitializerTypeCount")

        self.assertEqual(MLCompute.MLCDeviceTypeCPU, 0)
        self.assertEqual(MLCompute.MLCDeviceTypeGPU, 1)
        self.assertEqual(MLCompute.MLCDeviceTypeAny, 2)
        self.assertEqual(MLCompute.MLCDeviceTypeANE, 3)
        self.assertNotHasAttr(MLCompute, "MLCDeviceTypeCount")

        self.assertEqual(MLCompute.MLCGraphCompilationOptionsNone, 0x00)
        self.assertEqual(MLCompute.MLCGraphCompilationOptionsDebugLayers, 0x01)
        self.assertEqual(MLCompute.MLCGraphCompilationOptionsDisableLayerFusion, 0x02)
        self.assertEqual(MLCompute.MLCGraphCompilationOptionsLinkGraphs, 0x04)
        self.assertEqual(MLCompute.MLCGraphCompilationOptionsComputeAllGradients, 0x08)

        self.assertEqual(MLCompute.MLCExecutionOptionsNone, 0x00)
        self.assertEqual(
            MLCompute.MLCExecutionOptionsSkipWritingInputDataToDevice, 0x01
        )
        self.assertEqual(MLCompute.MLCExecutionOptionsSynchronous, 0x02)
        self.assertEqual(MLCompute.MLCExecutionOptionsProfiling, 0x04)
        self.assertEqual(MLCompute.MLCExecutionOptionsForwardForInference, 0x08)
        self.assertEqual(MLCompute.MLCExecutionOptionsPerLayerProfiling, 0x10)

        self.assertEqual(MLCompute.MLCGradientClippingTypeByValue, 0)
        self.assertEqual(MLCompute.MLCGradientClippingTypeByNorm, 1)
        self.assertEqual(MLCompute.MLCGradientClippingTypeByGlobalNorm, 2)

        self.assertEqual(MLCompute.MLCArithmeticOperationAdd, 0)
        self.assertEqual(MLCompute.MLCArithmeticOperationSubtract, 1)
        self.assertEqual(MLCompute.MLCArithmeticOperationMultiply, 2)
        self.assertEqual(MLCompute.MLCArithmeticOperationDivide, 3)
        self.assertEqual(MLCompute.MLCArithmeticOperationFloor, 4)
        self.assertEqual(MLCompute.MLCArithmeticOperationRound, 5)
        self.assertEqual(MLCompute.MLCArithmeticOperationCeil, 6)
        self.assertEqual(MLCompute.MLCArithmeticOperationSqrt, 7)
        self.assertEqual(MLCompute.MLCArithmeticOperationRsqrt, 8)
        self.assertEqual(MLCompute.MLCArithmeticOperationSin, 9)
        self.assertEqual(MLCompute.MLCArithmeticOperationCos, 10)
        self.assertEqual(MLCompute.MLCArithmeticOperationTan, 11)
        self.assertEqual(MLCompute.MLCArithmeticOperationAsin, 12)
        self.assertEqual(MLCompute.MLCArithmeticOperationAcos, 13)
        self.assertEqual(MLCompute.MLCArithmeticOperationAtan, 14)
        self.assertEqual(MLCompute.MLCArithmeticOperationSinh, 15)
        self.assertEqual(MLCompute.MLCArithmeticOperationCosh, 16)
        self.assertEqual(MLCompute.MLCArithmeticOperationTanh, 17)
        self.assertEqual(MLCompute.MLCArithmeticOperationAsinh, 18)
        self.assertEqual(MLCompute.MLCArithmeticOperationAcosh, 19)
        self.assertEqual(MLCompute.MLCArithmeticOperationAtanh, 20)
        self.assertEqual(MLCompute.MLCArithmeticOperationPow, 21)
        self.assertEqual(MLCompute.MLCArithmeticOperationExp, 22)
        self.assertEqual(MLCompute.MLCArithmeticOperationExp2, 23)
        self.assertEqual(MLCompute.MLCArithmeticOperationLog, 24)
        self.assertEqual(MLCompute.MLCArithmeticOperationLog2, 25)
        self.assertEqual(MLCompute.MLCArithmeticOperationMultiplyNoNaN, 26)
        self.assertEqual(MLCompute.MLCArithmeticOperationDivideNoNaN, 27)
        self.assertEqual(MLCompute.MLCArithmeticOperationMin, 28)
        self.assertEqual(MLCompute.MLCArithmeticOperationMax, 29)

        self.assertNotHasAttr(MLCompute, "MLCArithmeticOperationCount")

        self.assertEqual(MLCompute.MLCLossTypeMeanAbsoluteError, 0)
        self.assertEqual(MLCompute.MLCLossTypeMeanSquaredError, 1)
        self.assertEqual(MLCompute.MLCLossTypeSoftmaxCrossEntropy, 2)
        self.assertEqual(MLCompute.MLCLossTypeSigmoidCrossEntropy, 3)
        self.assertEqual(MLCompute.MLCLossTypeCategoricalCrossEntropy, 4)
        self.assertEqual(MLCompute.MLCLossTypeHinge, 5)
        self.assertEqual(MLCompute.MLCLossTypeHuber, 6)
        self.assertEqual(MLCompute.MLCLossTypeCosineDistance, 7)
        self.assertEqual(MLCompute.MLCLossTypeLog, 8)
        self.assertNotHasAttr(MLCompute, "MLCLossTypeCount")

        self.assertEqual(MLCompute.MLCActivationTypeNone, 0)
        self.assertEqual(MLCompute.MLCActivationTypeReLU, 1)
        self.assertEqual(MLCompute.MLCActivationTypeLinear, 2)
        self.assertEqual(MLCompute.MLCActivationTypeSigmoid, 3)
        self.assertEqual(MLCompute.MLCActivationTypeHardSigmoid, 4)
        self.assertEqual(MLCompute.MLCActivationTypeTanh, 5)
        self.assertEqual(MLCompute.MLCActivationTypeAbsolute, 6)
        self.assertEqual(MLCompute.MLCActivationTypeSoftPlus, 7)
        self.assertEqual(MLCompute.MLCActivationTypeSoftSign, 8)
        self.assertEqual(MLCompute.MLCActivationTypeELU, 9)
        self.assertEqual(MLCompute.MLCActivationTypeReLUN, 10)
        self.assertEqual(MLCompute.MLCActivationTypeLogSigmoid, 11)
        self.assertEqual(MLCompute.MLCActivationTypeSELU, 12)
        self.assertEqual(MLCompute.MLCActivationTypeCELU, 13)
        self.assertEqual(MLCompute.MLCActivationTypeHardShrink, 14)
        self.assertEqual(MLCompute.MLCActivationTypeSoftShrink, 15)
        self.assertEqual(MLCompute.MLCActivationTypeTanhShrink, 16)
        self.assertEqual(MLCompute.MLCActivationTypeThreshold, 17)
        self.assertEqual(MLCompute.MLCActivationTypeGELU, 18)
        self.assertEqual(MLCompute.MLCActivationTypeHardSwish, 19)
        self.assertEqual(MLCompute.MLCActivationTypeClamp, 20)

        self.assertNotHasAttr(MLCompute, "MLCActivationTypeCount")

        self.assertEqual(MLCompute.MLCConvolutionTypeStandard, 0)
        self.assertEqual(MLCompute.MLCConvolutionTypeTransposed, 1)
        self.assertEqual(MLCompute.MLCConvolutionTypeDepthwise, 2)

        self.assertEqual(MLCompute.MLCPaddingPolicySame, 0)
        self.assertEqual(MLCompute.MLCPaddingPolicyValid, 1)
        self.assertEqual(MLCompute.MLCPaddingPolicyUsePaddingSize, 2)

        self.assertEqual(MLCompute.MLCPaddingTypeZero, 0)
        self.assertEqual(MLCompute.MLCPaddingTypeReflect, 1)
        self.assertEqual(MLCompute.MLCPaddingTypeSymmetric, 2)
        self.assertEqual(MLCompute.MLCPaddingTypeConstant, 3)

        self.assertEqual(MLCompute.MLCPoolingTypeMax, 1)
        self.assertEqual(MLCompute.MLCPoolingTypeAverage, 2)
        self.assertEqual(MLCompute.MLCPoolingTypeL2Norm, 3)
        self.assertNotHasAttr(MLCompute, "MLCPoolingTypeCount")

        self.assertEqual(MLCompute.MLCReductionTypeNone, 0)
        self.assertEqual(MLCompute.MLCReductionTypeSum, 1)
        self.assertEqual(MLCompute.MLCReductionTypeMean, 2)
        self.assertEqual(MLCompute.MLCReductionTypeMax, 3)
        self.assertEqual(MLCompute.MLCReductionTypeMin, 4)
        self.assertEqual(MLCompute.MLCReductionTypeArgMax, 5)
        self.assertEqual(MLCompute.MLCReductionTypeArgMin, 6)
        self.assertEqual(MLCompute.MLCReductionTypeL1Norm, 7)
        self.assertEqual(MLCompute.MLCReductionTypeAny, 8)
        self.assertEqual(MLCompute.MLCReductionTypeAll, 9)

        self.assertNotHasAttr(MLCompute, "MLCReductionTypeCount")

        self.assertEqual(MLCompute.MLCRegularizationTypeNone, 0)
        self.assertEqual(MLCompute.MLCRegularizationTypeL1, 1)
        self.assertEqual(MLCompute.MLCRegularizationTypeL2, 2)

        self.assertEqual(MLCompute.MLCSampleModeNearest, 0)
        self.assertEqual(MLCompute.MLCSampleModeLinear, 1)

        self.assertEqual(MLCompute.MLCSoftmaxOperationSoftmax, 0)
        self.assertEqual(MLCompute.MLCSoftmaxOperationLogSoftmax, 1)

        self.assertEqual(MLCompute.MLCLSTMResultModeOutput, 0x00)
        self.assertEqual(MLCompute.MLCLSTMResultModeOutputAndStates, 0x01)

        self.assertEqual(MLCompute.MLCComparisonOperationEqual, 0)
        self.assertEqual(MLCompute.MLCComparisonOperationNotEqual, 1)
        self.assertEqual(MLCompute.MLCComparisonOperationLess, 2)
        self.assertEqual(MLCompute.MLCComparisonOperationGreater, 3)
        self.assertEqual(MLCompute.MLCComparisonOperationLessOrEqual, 4)
        self.assertEqual(MLCompute.MLCComparisonOperationGreaterOrEqual, 5)
        self.assertEqual(MLCompute.MLCComparisonOperationLogicalAND, 6)
        self.assertEqual(MLCompute.MLCComparisonOperationLogicalOR, 7)
        self.assertEqual(MLCompute.MLCComparisonOperationLogicalNOT, 8)
        self.assertEqual(MLCompute.MLCComparisonOperationLogicalNAND, 9)
        self.assertEqual(MLCompute.MLCComparisonOperationLogicalNOR, 10)
        self.assertEqual(MLCompute.MLCComparisonOperationLogicalXOR, 11)

        self.assertNotHasAttr(MLCompute, "MLCComparisonOperationCount")

    def test_functions(self):
        MLCompute.MLCActivationTypeDebugDescription
        MLCompute.MLCArithmeticOperationDebugDescription
        MLCompute.MLCPaddingPolicyDebugDescription
        MLCompute.MLCLossTypeDebugDescription
        MLCompute.MLCReductionTypeDebugDescription
        MLCompute.MLCPaddingTypeDebugDescription
        MLCompute.MLCConvolutionTypeDebugDescription
        MLCompute.MLCPoolingTypeDebugDescription
        MLCompute.MLCSoftmaxOperationDebugDescription
        MLCompute.MLCSampleModeDebugDescription
        MLCompute.MLCLSTMResultModeDebugDescription

    @min_os_level("11.3")
    def test_functions11_3(self):
        MLCompute.MLCComparisonOperationDebugDescription

    @min_os_level("12.0")
    def test_functions12_0(self):
        MLCompute.MLCGradientClippingTypeDebugDescription



from PyObjCTools.TestSupport import TestCase

import MLCompute


class TestMLCLSTMDescriptor(TestCase):
    def test_methods(self):
        self.assertResultIsBOOL(MLCompute.MLCLSTMDescriptor.usesBiases)
        self.assertResultIsBOOL(MLCompute.MLCLSTMDescriptor.batchFirst)
        self.assertResultIsBOOL(MLCompute.MLCLSTMDescriptor.isBidirectional)
        self.assertResultIsBOOL(MLCompute.MLCLSTMDescriptor.returnsSequences)

        self.assertArgIsBOOL(
            MLCompute.MLCLSTMDescriptor.descriptorWithInputSize_hiddenSize_layerCount_usesBiases_isBidirectional_dropout_,
            3,
        )
        self.assertArgIsBOOL(
            MLCompute.MLCLSTMDescriptor.descriptorWithInputSize_hiddenSize_layerCount_usesBiases_isBidirectional_dropout_,
            4,
        )

        self.assertArgIsBOOL(
            MLCompute.MLCLSTMDescriptor.descriptorWithInputSize_hiddenSize_layerCount_usesBiases_isBidirectional_dropout_,  # noqa: B950
            3,
        )
        self.assertArgIsBOOL(
            MLCompute.MLCLSTMDescriptor.descriptorWithInputSize_hiddenSize_layerCount_usesBiases_isBidirectional_dropout_,  # noqa: B950
            4,
        )

        self.assertArgIsBOOL(
            MLCompute.MLCLSTMDescriptor.descriptorWithInputSize_hiddenSize_layerCount_usesBiases_batchFirst_isBidirectional_returnsSequences_dropout_,  # noqa: B950
            3,
        )
        self.assertArgIsBOOL(
            MLCompute.MLCLSTMDescriptor.descriptorWithInputSize_hiddenSize_layerCount_usesBiases_batchFirst_isBidirectional_returnsSequences_dropout_,  # noqa: B950
            4,
        )
        self.assertArgIsBOOL(
            MLCompute.MLCLSTMDescriptor.descriptorWithInputSize_hiddenSize_layerCount_usesBiases_batchFirst_isBidirectional_returnsSequences_dropout_,  # noqa: B950
            5,
        )
        self.assertArgIsBOOL(
            MLCompute.MLCLSTMDescriptor.descriptorWithInputSize_hiddenSize_layerCount_usesBiases_batchFirst_isBidirectional_returnsSequences_dropout_,  # noqa: B950
            6,
        )

        self.assertArgIsBOOL(
            MLCompute.MLCLSTMDescriptor.descriptorWithInputSize_hiddenSize_layerCount_usesBiases_batchFirst_isBidirectional_returnsSequences_dropout_resultMode_,  # noqa: B950
            3,
        )
        self.assertArgIsBOOL(
            MLCompute.MLCLSTMDescriptor.descriptorWithInputSize_hiddenSize_layerCount_usesBiases_batchFirst_isBidirectional_returnsSequences_dropout_resultMode_,  # noqa: B950
            4,
        )
        self.assertArgIsBOOL(
            MLCompute.MLCLSTMDescriptor.descriptorWithInputSize_hiddenSize_layerCount_usesBiases_batchFirst_isBidirectional_returnsSequences_dropout_resultMode_,  # noqa: B950
            5,
        )
        self.assertArgIsBOOL(
            MLCompute.MLCLSTMDescriptor.descriptorWithInputSize_hiddenSize_layerCount_usesBiases_batchFirst_isBidirectional_returnsSequences_dropout_resultMode_,  # noqa: B950
            6,
        )
